import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { RPCError } from 'packages/core/rpc/error';

import { DiscoveryStartInfo } from '../../model';
import { DiscoveryService } from '../../service/discovery.service';

import {
  Setting,
  SettingSuccess,
  SettingFailure,
  ActionType,
} from './setting.action';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private discoveryService: DiscoveryService,
    private router: Router
  ) { }

  @Effect()
  signin$: Observable<Action> = this.actions$
    .ofType(ActionType.Setting)
    .map((action: Setting) => action.payload)
    .switchMap(payload => this.discoveryService.start(payload.payload))
    .map(discoveryStartInfo => {
      return new SettingSuccess(discoveryStartInfo);
    })
    .catch((error: RPCError) => {
      return of(new SettingFailure(error));
    });

    // .map((action: Signin) => action.payload)
    // .exhaustMap(payload =>
    //   this.memberService
    //     .signin(payload.email, payload.password)
    //     .map(member => {
    //       return new SigninSuccess(member);
    //     })
    //     .catch((error) => {
    //       return of(new SigninFailure(error.message));
    //     })
    // );

  @Effect({ dispatch: false })
  signinSuccess$ = this.actions$
    .ofType(ActionType.SettingSuccess)
    .do(() => this.router.navigate(['/']));

  @Effect({ dispatch: false })
  signinRedirect$ = this.actions$
    .ofType(ActionType.SettingRedirect)
    .do(authed => {
      this.router.navigate(['/login']);
    });

}
