import { Injectable } from '@angular/core';
import { Router } from '@angular/router';

import { Effect, Actions, ofType } from '@ngrx/effects';
import { Action } from '@ngrx/store';

import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/exhaustMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/take';

import { RPCError } from 'packages/core/rpc/error';

import { DomainMember } from 'packages/domain/model';

import { MetaCrawler } from '../../model';
import { CrawlerService } from '../../service/crawler.service';

import {
  ReadAll,
  ReadAllSuccess,
  ReadAllFailure,
  ActionType,
} from './readall.action';

@Injectable()
export class Effects {

  constructor(
    private actions$: Actions,
    private crawlerService: CrawlerService,
    private router: Router
  ) { }

  @Effect()
  signin$: Observable<Action> = this.actions$
    .ofType(ActionType.ReadAll)
    .map((action: ReadAll) => action.payload)
    .switchMap(payload => this.crawlerService.readAll())
    .map((metaCrawlerList: MetaCrawler[]) => {
      return new ReadAllSuccess(metaCrawlerList);
    })
    .catch((error: RPCError) => {
      return of(new ReadAllFailure(error));
    });

  @Effect({ dispatch: false })
  signinSuccess$ = this.actions$
    .ofType(ActionType.ReadAllSuccess)
    .do(() => this.router.navigate(['/']));

}
